/****************************************************************************

 This file is part of GLC-Player.
 Copyright (C) 2007-2008 Laurent Ribon (laumaya@users.sourceforge.net)
 Version 2.0.0, packaged on March 2009.

 http://www.glc-player.net

 GLC-Player is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 GLC-Player is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with GLC-Player; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

*****************************************************************************/

#include "OpenFileThread.h"
#include <GLC_FileFormatException>


OpenFileThread::OpenFileThread(GLC_Factory *pFactory)
: m_ModelId(0)
, m_pFactory(pFactory)
, m_pLoadingFile(NULL)
, m_pWorld(NULL)
, m_ErrorMsg()
, m_AttachedFileName()
{
	connect(m_pFactory, SIGNAL(currentQuantum(int)), this, SIGNAL(currentQuantum(int)));
}

OpenFileThread::~OpenFileThread()
{
	m_AttachedFileName.clear();
	if (NULL != m_pWorld)
	{
		delete m_pWorld;
	}
	if (NULL != m_pLoadingFile)
	{
		delete m_pLoadingFile;
	}
}

GLC_World OpenFileThread::getWorld()
{
	GLC_World result;
	if (NULL != m_pWorld)
	{
		result= (*m_pWorld);
		delete m_pWorld;
		m_pWorld= NULL;
	}
	return result;
}

GLC_uint OpenFileThread::getModelId() const
{
	return m_ModelId;
}

void OpenFileThread::setOpenFile(const GLC_uint id, QFile *openFile)
{
	m_ModelId= id;
	m_pLoadingFile= new QFile(openFile->fileName());
	m_AttachedFileName.clear();

}
void OpenFileThread::run()
{
	try
	{
		m_pWorld= m_pFactory->createWorld(*m_pLoadingFile, &m_AttachedFileName);
		m_ErrorMsg= "";
	}
	catch (GLC_FileFormatException &e)
	{
		switch (e.exceptionType())
		{
		case GLC_FileFormatException::FileNotFound :
			m_ErrorMsg= tr("File \"") + QFileInfo(m_pLoadingFile->fileName()).fileName() + tr("\" not found");
			break;
		case GLC_FileFormatException::FileNotSupported :
			m_ErrorMsg= tr("File not supported");
			break;
		case GLC_FileFormatException::WrongFileFormat :
			m_ErrorMsg= tr("File corrupted");
			break;
		case GLC_FileFormatException::NoMeshFound :
			m_ErrorMsg= tr("No mesh found");
			break;
		default :
			break;
		}

		if (NULL != m_pWorld)
		{
			delete m_pWorld;
			m_pWorld= NULL;
		}
		m_pLoadingFile->close();
		//m_ErrorMsg= e.what();
		emit loadError();
	}
	catch (std::exception &e)
	{
		if (NULL != m_pWorld)
		{
			delete m_pWorld;
			m_pWorld= NULL;
		}

		m_ErrorMsg= e.what();
		emit loadError();
	}

}

