/****************************************************************************

 This file is part of GLC-Player.
 Copyright (C) 2007-2008 Laurent Ribon (laumaya@users.sourceforge.net)
 Version 2.0.0, packaged on March 2009.

 http://www.glc-player.net

 GLC-Player is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 GLC-Player is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with GLC-Player; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

*****************************************************************************/

#include "AboutOpenglView.h"
#include <GLC_Exception>
#include <GLC_Factory>

AboutOpenglView::AboutOpenglView(QWidget * pParent)
: QGLWidget(pParent)
, m_GlView(this)
, m_pWorld(NULL)
, m_Light()
, m_MotionTimer()
, m_RotationVector(glc::Y_AXIS)
{
	m_Light.setPosition(1.0, 1.0, 1.0);

	// Set backroundImage
	m_GlView.loadBackGroundImage(":images/default_background.png");

	// Signal and slot connection
	connect(&m_MotionTimer, SIGNAL(timeout()), this, SLOT(rotateView()));
	QFile logoFile(":model3d/logo.obj");
	m_pWorld= GLC_Factory::instance()->createWorld(logoFile);

}

AboutOpenglView::~AboutOpenglView()
{
	delete m_pWorld;
	doneCurrent();
}

//////////////////////////////////////////////////////////////////////
// Public Set Functions
//////////////////////////////////////////////////////////////////////

// Reframe to the world
void AboutOpenglView::reframeAll()
{
	GLC_BoundingBox worldBoundingBox(m_pWorld->collection()->boundingBox());
	if (not worldBoundingBox.isEmpty())
	{
		m_GlView.reframe(worldBoundingBox);
		m_GlView.setDistMinAndMax(worldBoundingBox);
	}
}

//////////////////////////////////////////////////////////////////////
// Portected services Functions
//////////////////////////////////////////////////////////////////////

// Initialize OpenGL window
void AboutOpenglView::initializeGL()
{
	// Define position of the camera eye
	const GLC_Vector4d VectEye(1.0, 1.0, 1.0);
	// of the target
	const GLC_Vector4d VectTarget;
	// Up vector
	GLC_Vector4d VectUp(0.0, 1.0, 0.0);
	m_GlView.m_pViewCam->setCam(VectEye, VectTarget, VectUp);

	m_GlView.initGl();

	glEnable(GL_NORMALIZE);
	reframeAll();
	m_MotionTimer.start(33);
}

// paint in the opengl widget
void AboutOpenglView::paintGL()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();

	// Enable and execute lighting
	m_Light.enable();
	try
	{
		m_Light.glExecute();
		m_GlView.glExecuteCam();

		// Display normal object
		m_pWorld->glExecute(false);
	}
	catch (GLC_Exception &e)
	{
		qDebug() << e.what();
	}

}

// The widget as been resized
void AboutOpenglView::resizeGL(int width, int height)
{
	m_GlView.setWinGLSize(width, height);
}

//////////////////////////////////////////////////////////////////////
// Private slots Functions
//////////////////////////////////////////////////////////////////////
// Rotate the view
void AboutOpenglView::rotateView()
{
	GLC_Matrix4x4 RotationMatrix(m_RotationVector, glc::PI / 100.0);
	GLC_Camera savCam= (*m_GlView.m_pViewCam);
	m_GlView.m_pViewCam->translate(- savCam.getTarget());
	m_GlView.m_pViewCam->move(RotationMatrix);
	m_GlView.m_pViewCam->translate(savCam.getTarget());
	updateGL();
}
