/****************************************************************************

 This file is part of GLC-Player.
 Copyright (C) 2007-2008 Laurent Ribon (laumaya@users.sourceforge.net)
 Version 2.0.0, packaged on March 2009.

 http://www.glc-player.net

 GLC-Player is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 GLC-Player is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with GLC-Player; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

*****************************************************************************/

#include "EditLightDialog.h"

#include <GLC_Light>

#include <QtGui>

//////////////////////////////////////////////////////////////////////
// Constructor Destructor
//////////////////////////////////////////////////////////////////////
// Default constructor
EditLightDialog::EditLightDialog(GLC_Light* pLight, QWidget * parent)
    : QDialog(parent)
    , m_pLight(pLight)
{
    setupUi(this);
    initValues();
  	        
    // Signals and slot connection
    connect(editAmbient, SIGNAL(clicked()), this, SLOT(changeAmbientColor()));
    connect(editDiffuse, SIGNAL(clicked()), this, SLOT(changeDiffuseColor()));
    connect(editSpecular, SIGNAL(clicked()), this, SLOT(changeSpecularColor()));
    connect(buttonBox, SIGNAL(clicked(QAbstractButton*)), this, SLOT(buttonClicked(QAbstractButton*)));

}

//////////////////////////////////////////////////////////////////////
// Private Slots Functions
//////////////////////////////////////////////////////////////////////
// Change Ambient color
void EditLightDialog::changeAmbientColor()
{
	QColor color = QColorDialog::getColor(m_AmbientColor, this);
		if (color.isValid())
		{
			colorLabelAmbient->setPalette(QPalette(color));
			m_AmbientColor= color;			
		}
 	
}
// Change Diffuse color
void EditLightDialog::changeDiffuseColor()
{
	QColor color = QColorDialog::getColor(m_DiffuseColor, this);
		if (color.isValid())
		{
			colorLabelDiffuse->setPalette(QPalette(color));
			m_DiffuseColor= color;			
		}
 	
}

// Change Specular color
void EditLightDialog::changeSpecularColor()
{
	QColor color = QColorDialog::getColor(m_SpecularColor, this);
		if (color.isValid())
		{
			colorLabelSpecular->setPalette(QPalette(color));
			m_SpecularColor= color;			
		}
 	
}

// Button off botton box as been clicked
void EditLightDialog::buttonClicked(QAbstractButton* pButton)
{
	if (buttonBox->standardButton(pButton) == QDialogButtonBox::Apply)
	{
		updateValues();
		emit lightUpdated();
	}
}

void EditLightDialog::accept()
{
	updateValues();
	emit lightUpdated();
	QDialog::accept();	
}

//////////////////////////////////////////////////////////////////////
// Private services Functions
//////////////////////////////////////////////////////////////////////
// init background dialog values
void EditLightDialog::initValues()
{
	GLC_Vector4d lightPos(m_pLight->position());
	xValue->setValue(lightPos.X());
	yValue->setValue(lightPos.Y());
	zValue->setValue(lightPos.Z());
	
	m_AmbientColor= m_pLight->ambientColor();
	m_DiffuseColor= m_pLight->diffuseColor();
	m_SpecularColor= m_pLight->specularColor();
	
	colorLabelAmbient->setPalette(QPalette(m_AmbientColor));
	colorLabelDiffuse->setPalette(QPalette(m_DiffuseColor));
	colorLabelSpecular->setPalette(QPalette(m_SpecularColor));
}

// update light values
void EditLightDialog::updateValues()
{
	m_pLight->setPosition(xValue->value(), yValue->value(), zValue->value());	
	m_pLight->setAmbientColor(m_AmbientColor);
	m_pLight->setDiffuseColor(m_DiffuseColor);
	m_pLight->setSpecularColor(m_SpecularColor);	
}


