/****************************************************************************

 This file is part of GLC-Player.
 Copyright (C) 2007-2008 Laurent Ribon (laumaya@users.sourceforge.net)
 Version 2.0.0, packaged on March 2009.

 http://www.glc-player.net

 GLC-Player is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 GLC-Player is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with GLC-Player; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

 *****************************************************************************/

#include "SelectionProperty.h"
#include <GLC_Instance>
#include <GLC_Mesh2>

SelectionProperty::SelectionProperty(QAction* pShowHide, QAction* pAssignShader, QAction* pInstanceProperty, QWidget *parent)
: QWidget(parent)
, m_pSelections(NULL)
{
	setupUi(this);
	hideShowCmd->setDefaultAction(pShowHide);
	editPropertyCmd->setDefaultAction(pInstanceProperty);
	assignShaderCmd->setDefaultAction(pAssignShader);

	connect(reverseNormalsCmd, SIGNAL(clicked()), this, SLOT(reverseNormals()));

	numberOfObjectLabel->hide();
	numberOfObject->hide();
	editPropertyCmd->hide();
}

SelectionProperty::~SelectionProperty()
{
}

// Set the current selection
void SelectionProperty::setSelection(PointerNodeHash* pSelections)
{
	m_pSelections= pSelections;
	PointerNodeHash::iterator iEntry= m_pSelections->begin();

	int vertexNumber= 0;
	int faceNumber= 0;
	int subMaterialNumber= 0;

    while (iEntry != m_pSelections->constEnd())
    {
		GLC_Mesh2 *pCurrentMesh= dynamic_cast<GLC_Mesh2*>(iEntry.value()->getGeometry());
		Q_ASSERT(NULL != pCurrentMesh);
    	if (iEntry == m_pSelections->begin())
    	{
    		selectionName->setText(pCurrentMesh->name());
    	}
    	else
    	{
    		selectionName->clear();
    	}
    	vertexNumber+= pCurrentMesh->numberOfVertex();
    	faceNumber+= pCurrentMesh->numberOfFaces();
    	subMaterialNumber+= pCurrentMesh->numberOfMaterials();

		iEntry++;
    }
	numberOfVertex->setText(QString::number(vertexNumber));
	numberOfFace->setText(QString::number(faceNumber));
	numberOfSubMaterial->setText(QString::number(subMaterialNumber));
	if (m_pSelections->size() > 1)
	{
		numberOfObject->setText(QString::number(m_pSelections->size()));
		editPropertyCmd->hide();
		numberOfObjectLabel->show();
		numberOfObject->show();
	}
	else
	{
		numberOfObjectLabel->hide();
		numberOfObject->hide();
		editPropertyCmd->show();
	}
	reverseNormalsCmd->setEnabled(true);
	hideShowCmd->setEnabled(true);
}
// Unset the current selection
void SelectionProperty::unsetSelection()
{
	m_pSelections= NULL;
	selectionName->clear();
	numberOfVertex->clear();
	numberOfFace->clear();
	numberOfSubMaterial->clear();
	reverseNormalsCmd->setEnabled(false);
	hideShowCmd->setEnabled(false);

	numberOfObjectLabel->hide();
	numberOfObject->hide();
	editPropertyCmd->hide();
}
// Reverse selection normals
void SelectionProperty::reverseNormals()
{
	if (NULL != m_pSelections)
	{
		PointerNodeHash::iterator iEntry= m_pSelections->begin();
	    while (iEntry != m_pSelections->constEnd())
	    {
	    	GLC_Mesh2 *pCurrentMesh= dynamic_cast<GLC_Mesh2*>(iEntry.value()->getGeometry());
	    	pCurrentMesh->reverseNormal();
			iEntry++;
	    }
		emit updateView();
	}
}


